/*
 * MIT License
 *
 * Copyright (c) 2020 Alexey Edelev <semlanik@gmail.com>, Viktor Kopp <vifactor@gmail.com>
 *
 * This file is part of QtProtobuf project https://git.semlanik.org/semlanik/qtprotobuf
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#pragma once

#include <QMetaType>
#include <QtTest/QtTest>

#if defined(QT_QML_LIB)
    #define QT_PROTOBUF_PROPERTY_SUFFIX "_p"
#else
    #define QT_PROTOBUF_PROPERTY_SUFFIX
#endif

template<typename MessageType, typename PropertyType>
static void qProtobufAssertMessagePropertyRegistered(int fieldIndex, const char *propertyTypeName, const char *propertyName, bool skipMetatypeCheck = false)
{
    // TODO: there should be(?) a mapping avaialble: PropertyType -> propertyTypeName

    const int propertyNumber = MessageType::propertyOrdering.at(fieldIndex).qtProperty;
// TODO Qt6: Property type name check is disable because metatype system changes in Qt6.
// Q_PROPERTY returns non-aliased type for the aliases defined using the 'using' keyword.
//    QCOMPARE(QLatin1String(propertyTypeName), QLatin1String(MessageType::staticMetaObject.property(propertyNumber).typeName()));
    Q_UNUSED(propertyTypeName)
    if (!skipMetatypeCheck) {
        QCOMPARE(qMetaTypeId<PropertyType>(), MessageType::staticMetaObject.property(propertyNumber).userType());
    }
    QCOMPARE(QLatin1String(propertyName), QLatin1String(MessageType::staticMetaObject.property(propertyNumber).name()));
}
