/*
 * MIT License
 *
 * Copyright (c) 2022 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of QtProtobuf project https://git.semlanik.org/semlanik/qtprotobuf
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "serverlauncher.h"

#include <QDebug>

ServerLauncher::ServerLauncher()
{

}

ServerLauncher::~ServerLauncher()
{
    m_serverProcess.terminate();
    if(!m_serverProcess.waitForFinished(5000)) {
        m_serverProcess.kill();
    }
}

void ServerLauncher::start()
{
    m_serverStarupWaiter = new QEventLoop;
    if (m_serverProcess.state() != QProcess::NotRunning)
        qDebug() << "Server process is already running";

    m_serverProcess.setProgram(QT_GRPC_SERVER_EXECUTABLE);
    m_serverProcess.start();
    m_serverProcess.waitForStarted();

    connect(&m_serverProcess, &QProcess::readyReadStandardError, this, [this]{
        if (m_serverStarupWaiter != nullptr && m_serverStarupWaiter->isRunning())
            m_serverStarupWaiter->quit();
    });
    connect(&m_serverProcess, &QProcess::readyReadStandardOutput, this, [this]{
        if (m_serverStarupWaiter != nullptr && m_serverStarupWaiter->isRunning())
            m_serverStarupWaiter->quit();
    });

    connect(&m_serverProcess, &QProcess::readyReadStandardError, this, [this]{
        qDebug() << ":" << "SERVER_ERROR" << m_serverProcess.readAllStandardError();
    });
    connect(&m_serverProcess, &QProcess::readyReadStandardOutput, this, [this]{
        qDebug() << ":" << "SERVER_MESSAGE" << m_serverProcess.readAllStandardOutput();
    });

    m_serverStarupWaiter->exec();
    delete m_serverStarupWaiter;
    m_serverStarupWaiter = nullptr;
    qDebug() << "Server started";
}
