/*
 * MIT License
 *
 * Copyright (c) 2022 Alexey Edelev <semlanik@gmail.com>, Viktor Kopp <vifactor@gmail.com>
 *
 * This file is part of QtProtobuf project https://git.semlanik.org/semlanik/qtprotobuf
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#pragma once

#include "basicmessages.qpb.h"
#include "mapmessages.qpb.h"

#include <QObject>

#include <QtTest/QtTest>
#include <QProtobufSerializer>

#include <limits>

class QtProtobufMapTypesDeserializationTest : public QObject
{
    Q_OBJECT
public:
    QtProtobufMapTypesDeserializationTest()
    {
        qRegisterProtobufTypes();
    }
private slots:
    void init()
    {
        serializer.reset(new QProtobufSerializer);
    }

    void SimpleFixed32StringMapDeserializeTest();
    void SimpleSFixed32StringMapDeserializeTest();
    void SimpleInt32StringMapDeserializeTest();
    void SimpleSInt32StringMapDeserializeTest();
    void SimpleUInt32StringMapDeserializeTest();
    void SimpleFixed64StringMapDeserializeTest();
    void SimpleSFixed64StringMapDeserializeTest();
    void SimpleInt64StringMapDeserializeTest();
    void SimpleSInt64StringMapDeserializeTest();
    void SimpleUInt64StringMapDeserializeTest();
    void SimpleStringStringMapDeserializeTest();

    void SimpleFixed32ComplexMapDeserializeTest();
    void SimpleSFixed32ComplexMapDeserializeTest();
    void SimpleInt32ComplexMapDeserializeTest();
    void SimpleSInt32ComplexMapDeserializeTest();
    void SimpleUInt32ComplexMapDeserializeTest();
    void SimpleFixed64ComplexMapDeserializeTest();
    void SimpleSFixed64ComplexMapDeserializeTest();
    void SimpleInt64ComplexMapDeserializeTest();
    void SimpleSInt64ComplexMapDeserializeTest();
    void SimpleUInt64ComplexMapDeserializeTest();
    void SimpleStringComplexMapDeserializeTest();
    void SimpleUInt64ComplexMapInvalidLengthDeserializeTest();
    void SimpleStringComplexMapInvalidLengthDeserializeTest();
    void SimpleUInt64ComplexMapCorruptedDeserializeTest();
private:
    std::unique_ptr<QProtobufSerializer> serializer;
};
