/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of QtProtobuf project https://git.semlanik.org/semlanik/qtprotobuf
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <QtTest/QtTest>

#include <QVariant>
#include <qtprotobuftypes.h>

class QtProtobufConverterTest : public QObject
{
    Q_OBJECT
public:
    QtProtobufConverterTest()
    {
        qRegisterProtobufTypes();
    }
private slots:
    void TestFromTypeConverters();
    void TestToTypeConverters();
};

void QtProtobufConverterTest::TestFromTypeConverters()
{
    QVariant testVariant;

    testVariant.setValue<uint32_t>(42);
    QCOMPARE(42, testVariant.value<QtProtobuf::fixed32>()._t);

    testVariant.setValue<uint64_t>(43);
    QCOMPARE(43, testVariant.value<QtProtobuf::fixed64>()._t);

    testVariant.setValue<int32_t>(44);
    QCOMPARE(44, testVariant.value<QtProtobuf::sfixed32>()._t);

    testVariant.setValue<int64_t>(45);
    QCOMPARE(45, testVariant.value<QtProtobuf::sfixed64>()._t);

    testVariant.setValue<int32_t>(46);
    QCOMPARE(46, testVariant.value<QtProtobuf::int32>()._t);

    testVariant.setValue<int64_t>(47);
    QCOMPARE(47, testVariant.value<QtProtobuf::int64>()._t);
}

void QtProtobufConverterTest::TestToTypeConverters()
{
    bool ok = false;
    QVariant testVariant;
    testVariant.setValue<QtProtobuf::fixed32>({42});
    QCOMPARE(42, testVariant.toUInt(&ok));
    QVERIFY(ok);

    ok = false;
    testVariant.setValue<QtProtobuf::fixed64>({43});
    QCOMPARE(43, testVariant.toULongLong(&ok));
    QVERIFY(ok);

    ok = false;
    testVariant.setValue<QtProtobuf::sfixed32>({44});
    QCOMPARE(44, testVariant.toInt(&ok));
    QVERIFY(ok);

    ok = false;
    testVariant.setValue<QtProtobuf::sfixed64>({45});
    QCOMPARE(45, testVariant.toLongLong(&ok));
    QVERIFY(ok);

    ok = false;
    testVariant.setValue<QtProtobuf::int32>({46});
    QCOMPARE(46, testVariant.toInt(&ok));
    QVERIFY(ok);

    ok = false;
    testVariant.setValue<QtProtobuf::int64>({47});
    QCOMPARE(47, testVariant.toLongLong(&ok));
    QVERIFY(ok);
}

QTEST_MAIN(QtProtobufConverterTest)
#include "converterstest.moc"
