/*
 * MIT License
 *
 * Copyright (c) 2022 Alexey Edelev <semlanik@gmail.com>, Viktor Kopp <vifactor@gmail.com>
 *
 * This file is part of QtProtobuf project https://git.semlanik.org/semlanik/qtprotobuf
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "enummessages.qpb.h"

#include <QtTest/QtTest>
#include <QProtobufSerializer>

class QtProtobufEnumTypesDeserializationTest : public QObject
{
    Q_OBJECT
public:
    QtProtobufEnumTypesDeserializationTest()
    {
        qRegisterProtobufTypes();
    }
private slots:
    void init()
    {
        m_serializer.reset(new QProtobufSerializer);
    }
    void SimpleEnumMessageDeserializeTest();
    void RepeatedEnumMessageTest();
private:
    std::unique_ptr<QProtobufSerializer> m_serializer;
};

using namespace qtprotobufnamespace::tests;

void QtProtobufEnumTypesDeserializationTest::SimpleEnumMessageDeserializeTest()
{
    SimpleEnumMessage test;
    test.deserialize(m_serializer.get(), QByteArray::fromHex("0803"));
    QCOMPARE(test.localEnum(), SimpleEnumMessage::LOCAL_ENUM_VALUE3);
}

void QtProtobufEnumTypesDeserializationTest::RepeatedEnumMessageTest()
{
    RepeatedEnumMessage msg;

    msg.deserialize(m_serializer.get(), QByteArray());
    QVERIFY(msg.localEnumList().isEmpty());

    msg.deserialize(m_serializer.get(), QByteArray::fromHex("0a06000102010203"));
    QVERIFY((msg.localEnumList() == RepeatedEnumMessage::LocalEnumRepeated {RepeatedEnumMessage::LOCAL_ENUM_VALUE0,
                RepeatedEnumMessage::LOCAL_ENUM_VALUE1,
                RepeatedEnumMessage::LOCAL_ENUM_VALUE2,
                RepeatedEnumMessage::LOCAL_ENUM_VALUE1,
                RepeatedEnumMessage::LOCAL_ENUM_VALUE2,
                RepeatedEnumMessage::LOCAL_ENUM_VALUE3}));
}

QTEST_MAIN(QtProtobufEnumTypesDeserializationTest)
#include "deserializationenumtypes.moc"
