/*
 * MIT License
 *
 * Copyright (c) 2022 Alexey Edelev <semlanik@gmail.com>, Viktor Kopp <vifactor@gmail.com>
 *
 * This file is part of QtProtobuf project https://git.semlanik.org/semlanik/qtprotobuf
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "enummessages.qpb.h"

#include <QtTest/QtTest>
#include <QProtobufJsonSerializer>

class QtProtobufEnumTypesJsonDeserializationTest : public QObject
{
    Q_OBJECT
public:
    QtProtobufEnumTypesJsonDeserializationTest()
    {
        qRegisterProtobufTypes();
    }
private slots:
    void init()
    {
        m_serializer.reset(new QProtobufJsonSerializer);
    }
    void SimpleEnumMessageDeserializeTest();
    void RepeatedEnumMessageTest();
private:
    std::unique_ptr<QProtobufJsonSerializer> m_serializer;
};

using namespace qtprotobufnamespace::tests;

void QtProtobufEnumTypesJsonDeserializationTest::SimpleEnumMessageDeserializeTest()
{
    SimpleEnumMessage test;
    test.deserialize(m_serializer.get(), "{\"localEnum\":\"LOCAL_ENUM_VALUE2\"}");
    QCOMPARE(test.localEnum(), SimpleEnumMessage::LOCAL_ENUM_VALUE2);
}

void QtProtobufEnumTypesJsonDeserializationTest::RepeatedEnumMessageTest()
{
    RepeatedEnumMessage test;
    test.deserialize(m_serializer.get(), "{\"localEnumList\":[\"LOCAL_ENUM_VALUE0\",\"LOCAL_ENUM_VALUE1\",\"LOCAL_ENUM_VALUE2\",\"LOCAL_ENUM_VALUE1\",\"LOCAL_ENUM_VALUE2\",\"LOCAL_ENUM_VALUE3\"]}");
    QVERIFY(test.localEnumList() == RepeatedEnumMessage::LocalEnumRepeated({RepeatedEnumMessage::LOCAL_ENUM_VALUE0,
                                       RepeatedEnumMessage::LOCAL_ENUM_VALUE1,
                                       RepeatedEnumMessage::LOCAL_ENUM_VALUE2,
                                       RepeatedEnumMessage::LOCAL_ENUM_VALUE1,
                                       RepeatedEnumMessage::LOCAL_ENUM_VALUE2,
                                       RepeatedEnumMessage::LOCAL_ENUM_VALUE3}));
    test.deserialize(m_serializer.get(), "{\"localEnumList\":[]}");
    QVERIFY(test.localEnumList().isEmpty());

    test.deserialize(m_serializer.get(), "{\"localEnumList\":[\"LOCAL_ENUM_VALUE0\",\"LOCAL_ENUM_VALUE1\",null,\"LOCAL_ENUM_VALUE1\",\"LOCAL_ENUM_VALUE2\",\"LOCAL_ENUM_VALUE3\"]}");
    QVERIFY(test.localEnumList() == RepeatedEnumMessage::LocalEnumRepeated({RepeatedEnumMessage::LOCAL_ENUM_VALUE0,
                                       RepeatedEnumMessage::LOCAL_ENUM_VALUE1,
                                       RepeatedEnumMessage::LOCAL_ENUM_VALUE0,
                                       RepeatedEnumMessage::LOCAL_ENUM_VALUE1,
                                       RepeatedEnumMessage::LOCAL_ENUM_VALUE2,
                                       RepeatedEnumMessage::LOCAL_ENUM_VALUE3}));
    test.deserialize(m_serializer.get(), "{\"localEnumList\":null}");
    QVERIFY(test.localEnumList().isEmpty());
}

QTEST_MAIN(QtProtobufEnumTypesJsonDeserializationTest)
#include "deserializationenumtypes.moc"
