/*
 * MIT License
 *
 * Copyright (c) 2022 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of QtProtobuf project https://git.semlanik.org/semlanik/qtprotobuf
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "enummessages.qpb.h"

#include <QtTest/QtTest>
#include <QProtobufSerializer>

class QtProtobufEnumTypesSerializationTest : public QObject
{
    Q_OBJECT
public:
    QtProtobufEnumTypesSerializationTest() {
        qRegisterProtobufTypes();
    }

    void SimpleEnumMessageSerializeTest();
    void RepeatedEnumMessageTest();

private slots:
    void init() {
        m_serializer.reset(new QProtobufSerializer);
    }
private:
    std::unique_ptr<QProtobufSerializer> m_serializer;
};

using namespace qtprotobufnamespace::tests;

void QtProtobufEnumTypesSerializationTest::SimpleEnumMessageSerializeTest()
{
    SimpleEnumMessage test;
    test.setLocalEnum(SimpleEnumMessage::LOCAL_ENUM_VALUE2);
    QByteArray result = test.serialize(m_serializer.get());
    QCOMPARE(2, result.size());
    QCOMPARE(result.toHex().toStdString().c_str(), "0802");
}

void QtProtobufEnumTypesSerializationTest::RepeatedEnumMessageTest()
{
    RepeatedEnumMessage msg;

    msg.setLocalEnumList({RepeatedEnumMessage::LOCAL_ENUM_VALUE0,
                          RepeatedEnumMessage::LOCAL_ENUM_VALUE1,
                          RepeatedEnumMessage::LOCAL_ENUM_VALUE2,
                          RepeatedEnumMessage::LOCAL_ENUM_VALUE1,
                          RepeatedEnumMessage::LOCAL_ENUM_VALUE2,
                          RepeatedEnumMessage::LOCAL_ENUM_VALUE3});
    QByteArray result = msg.serialize(m_serializer.get());
    QCOMPARE(result.toHex().toStdString().c_str(),
                 "0a06000102010203");
    msg.setLocalEnumList({});
    result = msg.serialize(m_serializer.get());
    QCOMPARE(result.toHex().toStdString().c_str(),
                 "");
}

QTEST_MAIN(QtProtobufEnumTypesSerializationTest)
#include "serializationenumtypes.moc"
